unit Retrieve;

{   Wall Weather Ver 1.0                                    }
{                                                           }
{   This application Require the WEB component developed    }
{   by Emanuele Cipolloni at Leviathan@dlep1.itg.ti.com     }
{                                                           }

interface

uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, ExtCtrls, StdCtrls, INIFiles, Web, Gif2Bmp, Globals,
  Setting;

type
  TWeather = class(TForm)
    Web1: TWeb;
    Timer1: TTimer;
    Panel1: TPanel;
    DoRetrieve: TButton;
    DCounter: TLabel;
    DTitle: TLabel;
    Label3: TLabel;
    Status: TLabel;
    Panel2: TPanel;
    Monitor: TImage;
    RetAbort: TButton;
    Config: TButton;
    Quit: TButton;
    procedure Timer1Timer(Sender: TObject);
    procedure Web1Connect(Sender: TObject);
    procedure Web1Read(Sender: TObject);
    procedure Web1Disconnect(Sender: TObject);
    procedure DoRetrieveClick(Sender: TObject);
    procedure ConfigClick(Sender: TObject);
    procedure QuitClick(Sender: TObject);
    procedure RetAbortClick(Sender: TObject);
  private
    { Private declarations }
  public
    { Public declarations }
  end;

var
  Weather: TWeather;
  TimeCounter : integer;

procedure GetImage;
procedure Transform;

implementation

{$R *.DFM}

procedure TWeather.Timer1Timer(Sender: TObject);
begin
     { Wait 'til fixed Idle time elapsed then trigger image retrieving}
     Inc(TimeCounter);
     if TimeCounter > Idle then
     begin
        Weather.RetAbort.Enabled := True;
        GetImage;
     end;

end;

procedure TWeather.Web1Connect(Sender: TObject);
begin
     Status.Caption := 'Connecting...';
end;

procedure TWeather.Web1Read(Sender: TObject);
begin
     { Update retrieving info }
     DTitle.Caption := 'Size:';
     Status.Caption := 'Retrieving...';
     DCounter.Caption := IntToStr(Web1.size);
end;

procedure TWeather.Web1Disconnect(Sender: TObject);
begin
     Weather.RetAbort.Enabled := False;
     Transform;
end;

procedure TWeather.DoRetrieveClick(Sender: TObject);
begin
     Weather.RetAbort.Enabled := True;
     GetImage;
end;

procedure Transform;
var
   Buffer : array[0..40] of Char;    {To hold temporary path name in null format }
   Source, Matte : TBitmap;          {To hold image processing data }
   r1,g1,b1,
   r2,g2,b2 : single;                {To hold single color component values: Land and Sea }
   x,y : integer;                    {To hold pixel coordionates in source image }
   Luminance : integer;
   SourceColor,
   MatteColor,
   ResultColor,
   LandLastColor,
   SeaLastColor,
   LandFirstColor,
   SeaFirstColor : TColorRef;         {To hold color reference to perform recoloring }
   Gif : TGif;                        {To hold a TGOF object for GIF->BMP conversion }
begin
     if Weather.Web1.Size > 0 then
     begin
          Weather.Status.Caption := 'Converting...';
          Weather.DTitle.Caption := '';
          Weather.DCounter.Caption := '';
          Application.ProcessMessages;

          if ReColor then
          begin
               Gif := TGif.Create;
               Gif.Convert(WorkDir + 'europe.gif', WorkDir + 'act.bmp');
               Gif.Free;

               { This will hold the orginal image}
               Source := TBitmap.Create;
               with Source do
               begin
                    Width := 800;
                    Height := 628;          {+28 to eliminate the printing on top of the image }
                    Monochrome := False;
                    LoadFromFile(WorkDir + 'act.bmp');
               end;

               with Weather do
               begin
                    Status.Caption := 'Processing...';
                    DTitle.Caption := 'Line:';
                    DCounter.Caption := '';
               end;
               Application.ProcessMessages;

               { This will hold the matte bitmap }
               Matte := TBitmap.Create;
               with Matte do
               begin
                    Width := 800;
                    Height := 600;
                    Monochrome := false;
                    LoadFromFile(WorkDir + 'matte.bmp');
               end;

               Weather.Monitor.Picture.LoadFromFile(WorkDir + 'matte.bmp');


               Application.ProcessMessages;

               SeaFirstColor := GetNearestColor(Weather.Monitor.Canvas.Handle,RGB(129,88,10));
               SeaLastColor := GetNearestColor(Weather.Monitor.Canvas.Handle,RGB(253,247,235));

               LandFirstColor := GetNearestColor(Weather.Monitor.Canvas.Handle,RGB(47,47,212));
               LandLastColor := GetNearestColor(Weather.Monitor.Canvas.Handle,RGB(235,235,235));

               r1 := (GetRValue(SeaLastColor) - GetRValue(SeaFirstColor)) / 127;
               g1 := (GetGValue(SeaLastColor) - GetGValue(SeaFirstColor)) / 127;
               b1 := (GetBValue(SeaLastColor) - GetBValue(SeaFirstColor)) / 127;

               r2 := (GetRValue(LandLastColor) - GetRValue(LandFirstColor)) / 127;
               g2 := (GetGValue(LandLastColor) - GetGValue(LandFirstColor)) / 127;
               b2 := (GetBValue(LandLastColor) - GetBValue(LandFirstColor)) / 127;

               for x:= 1 to 800 do
               begin
                    Weather.DCounter.Caption := IntToStr(x);
                    Application.ProcessMessages;
                    for y := 1 to 600 do
                    begin
                         SourceColor := Source.Canvas.Pixels[x, y + 28];
                         MatteColor := Matte.Canvas.Pixels[x, y];

                         if MatteColor <> 0 then
                         begin
                              Luminance := GetRValue(SourceColor);
                              Luminance := Luminance div 2;

                              if MatteColor = SeaFirstColor then
                              begin
                                   ResultColor := Rgb(
                                             Trunc(GetRValue(SeaFirstColor) + Luminance * r1),
                                             Trunc(GetGValue(SeaFirstColor) + Luminance * g1),
                                             Trunc(GetBValue(SeaFirstColor) + Luminance * b1));
                                   Matte.Canvas.Pixels[x,y] := ResultColor;
                              end
                              else
                              begin
                                   ResultColor := Rgb(
                                             Trunc(GetRValue(LandFirstColor) + Luminance * r2),
                                             Trunc(GetgValue(LandFirstColor) + Luminance * g2),
                                             Trunc(GetBValue(LandFirstColor) + Luminance * b2));
                                   Matte.canvas.pixels[x,y] := ResultColor
                              end;
                         end
                         else
                             Matte.Canvas.Pixels[x,y] := 1;
                         Application.ProcessMessages;
                    end;
                    if (x mod 20) = 0 then
                    begin
                         Weather.Monitor.Canvas.CopyRect(Rect(0,0,800,600),
                                                         Matte.Canvas,
                                                         Rect(0,0,800,600));
                         Application.ProcessMessages;
                    end;
               end;

               Weather.Monitor.Canvas.CopyRect(Rect(0,0,800,600),
                                               Matte.Canvas,
                                               Rect(0,0,800,600));
               Application.ProcessMessages;

               Weather.Status.Caption := 'Saving...';
               Weather.DTitle.Caption := '';
               Weather.DCounter.Caption := '';
               Application.ProcessMessages;

               Matte.SaveToFile(WorkDir + 'europe.bmp');

               Application.ProcessMessages;

               Source.Free;
               Matte.Free;
          end
          else
          begin
               Gif := TGif.Create;
               Gif.Convert(WorkDir + 'europe.gif', WorkDir + 'europe.bmp');
               Gif.Free;
          end;

          if Stamp then
          begin
               StrPCopy(Buffer,WorkDir + 'europe.bmp');
               SystemParametersInfo(SPI_SETDESKWALLPAPER,0,@Buffer,0);
          end;
     end;

          Weather.Status.Caption := 'Waiting';

          Weather.Timer1.Enabled := true;
          Weather.DoRetrieve.Enabled := true;
end;

procedure GetImage;
begin
     TimeCounter := 0;

     with Weather do
     begin
          Timer1.Enabled := False;
          DoRetrieve.Enabled := False;
          Status.Caption := 'Initializing...';
     end;

     with Weather.Web1 do
     begin

          if UProxy then
          begin
               Weather.Web1.Proxy := ProxyName;
               Weather.Web1.ProxyPort := StrToInt(ProxyPortNumber);
               Weather.Web1.UseProxy := UProxy;
          end;

          Weather.Web1.Method := 'GET';

          Weather.Web1.UseFileStream := true;
          Weather.Web1.FileStreamName := WorkDir + 'europe.gif';
          Weather.Web1.URL := 'http://rs560.cl.msu.edu/weather/d2.gif';

          Weather.Web1.UseStream := True;
     end;

     Weather.Web1.Retrieve;
end;

procedure TWeather.ConfigClick(Sender: TObject);
begin
     with Settings, WIni do
     begin
          UseProxy.Checked := ReadBool('Settings','UseProxy',False);
          PName.Text := ReadString('Settings','Proxy','');
          PPort.Text := ReadString('Settings','ProxyPort','');
          WDir.Text := ReadString('Settings','WorkDir','');
          ICount.Value := ReadInteger('Settings','Idle',30);
          DoStamp.Checked := ReadBool('Settings','Stamp',True);
          DoRecolor.Checked := ReadBool('Settings','Recolor',True);
     end;

     Settings.ShowModal;
end;

procedure TWeather.QuitClick(Sender: TObject);
begin
     Application.Terminate;
end;

procedure TWeather.RetAbortClick(Sender: TObject);
begin
     Weather.Web1.Abort;
     Weather.RetAbort.Enabled := False;
end;

initialization
TimeCounter := 0;
with WIni do
begin
     UProxy := ReadBool('Settings','UseProxy',False);
     ProxyName := ReadString('Settings','Proxy','');
     ProxyPortNumber := ReadString('Settings','ProxyPort','');
     WorkDir := ReadString('Settings','WorkDir','');
     Idle := ReadInteger('Settings','Idle',30);
     Stamp := ReadBool('Settings','Stamp',True);
     Recolor := ReadBool('Settings','Recolor',True);
end;

end.
